/*
    Copyright (  C ) 2003 Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation;
    version 2 of the License.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/

#ifndef ARTS_KLEVELMETER_TEMPLATE_H
#define ARTS_KLEVELMETER_TEMPLATE_H

#include <qwidget.h>

#include <math.h>

#include "klevelmeter_impl.h"


class KLevelMeter_Template : public QWidget {
   Q_OBJECT
public:
	Arts::KLevelMeter_impl* _impl;

	KLevelMeter_Template( Arts::KLevelMeter_impl* impl, QWidget* p, long substyle, long count, float _dbmin, float _dbmax )
	  : QWidget( p )
	  , _impl( impl )
	  , _count( count )
	  , dbmax( _dbmax )
	  , dbmin( _dbmin )
	  , nilline( 3/4.0 )
	{}

	virtual void invalue( float, float =0 ) =0;

	virtual void volume( float ) =0;

	virtual void substyle( long ) {}
	virtual long substyle() { return 0; }

	virtual void count( long ) {}
	virtual long count() { return 0; }
	long _count;

	float dbmax, dbmin;
	/**
		Logarithmic/decimal valuation:
		p = ampfactor; d = dezibel
		p = 10^(  d/10 )
		db = 10*lg(  p )
	*/
	float amptodb( float p ) {
		float db = 10*log10( p );
		if ( db < dbmin ) db = dbmin;
		if ( db > dbmax ) db = dbmax;
		return db;
	}
	float dbtoamp( float db ) {
		float amp = pow( 10, db/10 );
		if ( amp <= pow( 10, dbmin/10 ) ) amp = 0;
		return amp;
	}
	/// Normalizes a dezibel value.
	float dbtondb( float db ) {
		return ( db -dbmin )/( dbmax-dbmin );
	}
	/// Normalizes a volume to a logarithmic value between 0 and 1.
	float dbtovol( float db ) {
		return ( db -dbmin )/( 0-dbmin );
	}
	/// Unnormalizes a dezibel value.
	float ndbtodb( float ndb ) {
		return ( ndb * ( dbmax-dbmin ) +dbmin );
	}
	float nilline;
	/// Gives the colors between green and red
	QColor color( float n ) {
		return QColor( int( ( n<=nilline )?255*( 1/nilline )*n:255 ),
		               int( ( n<=1 && n>nilline )?255-255*( 1/nilline )*( n-nilline ):( ( n>1 )?0:255 ) ),
		               0 );
	}
};

#endif
// vim: sw=4 ts=4
