/*

    Copyright (C) 2003 Nikolas Zimmermann <wildfox@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/

#ifndef READERWRITER_H
#define READERWRITER_H

#include <qobject.h>
#include <stdio.h>

class KURL;

class ReaderWriter
{
public:
	ReaderWriter();
	~ReaderWriter();

	typedef enum
	{
		WRITE = 0,
		READ = 1
	} Mode;
	
	int fd() const;
	void setFD(Mode m, int fd);
	
	FILE *openStream() const;

	bool write16Number(Q_UINT16 number) const; // Write Q_UINT16 into file.
	bool write32Number(Q_UINT32 number) const; // Write Q_UINT32 into file.
	bool writeString(const QString &string) const; // Write QString into file.

	bool eof() const;
	bool setup(int fd, Mode m);
	bool setup(const KURL &file, Mode m);

protected:
	// Used by Reader + Writer
	char *m_buf; // Pointer to data buffer
	size_t m_size; // Size of buffer
	size_t m_count; // Cound read in to buffer
	size_t m_pos; // Position in buffer

	bool m_eof; // End of file marker

	int m_signalSize; // Word length of signal data

private:
	FILE *m_stream;
	int m_fd;
};

#endif
